@@ -6,7 +6,7 @@ import borgmatic.actions.restore as module
 
 def test_get_configured_database_matches_database_by_name():
     assert module.get_configured_database(
-        hooks={
+        config={
             'other_databases': [{'name': 'other'}],
             'postgresql_databases': [{'name': 'foo'}, {'name': 'bar'}],
         },
@@ -18,7 +18,7 @@ def test_get_configured_database_matches_database_by_name():
 
 def test_get_configured_database_matches_nothing_when_database_name_not_configured():
     assert module.get_configured_database(
-        hooks={'postgresql_databases': [{'name': 'foo'}, {'name': 'bar'}]},
+        config={'postgresql_databases': [{'name': 'foo'}, {'name': 'bar'}]},
         archive_database_names={'postgresql_databases': ['foo']},
         hook_name='postgresql_databases',
         database_name='quux',
@@ -27,7 +27,7 @@ def test_get_configured_database_matches_nothing_when_database_name_not_configur
 
 def test_get_configured_database_matches_nothing_when_database_name_not_in_archive():
     assert module.get_configured_database(
-        hooks={'postgresql_databases': [{'name': 'foo'}, {'name': 'bar'}]},
+        config={'postgresql_databases': [{'name': 'foo'}, {'name': 'bar'}]},
         archive_database_names={'postgresql_databases': ['bar']},
         hook_name='postgresql_databases',
         database_name='foo',
@@ -36,7 +36,7 @@ def test_get_configured_database_matches_nothing_when_database_name_not_in_archi
 
 def test_get_configured_database_matches_database_by_configuration_database_name():
     assert module.get_configured_database(
-        hooks={'postgresql_databases': [{'name': 'all'}, {'name': 'bar'}]},
+        config={'postgresql_databases': [{'name': 'all'}, {'name': 'bar'}]},
         archive_database_names={'postgresql_databases': ['foo']},
         hook_name='postgresql_databases',
         database_name='foo',
@@ -46,7 +46,7 @@ def test_get_configured_database_matches_database_by_configuration_database_name
 
 def test_get_configured_database_with_unspecified_hook_matches_database_by_name():
     assert module.get_configured_database(
-        hooks={
+        config={
             'other_databases': [{'name': 'other'}],
             'postgresql_databases': [{'name': 'foo'}, {'name': 'bar'}],
         },
@@ -69,9 +69,9 @@ def test_collect_archive_database_names_parses_archive_paths():
     archive_database_names = module.collect_archive_database_names(
         repository={'path': 'repo'},
         archive='archive',
-        location={'borgmatic_source_directory': '.borgmatic'},
-        storage=flexmock(),
+        config={'borgmatic_source_directory': '.borgmatic'},
         local_borg_version=flexmock(),
+        global_arguments=flexmock(log_json=False),
         local_path=flexmock(),
         remote_path=flexmock(),
     )
@@ -94,9 +94,9 @@ def test_collect_archive_database_names_parses_directory_format_archive_paths():
     archive_database_names = module.collect_archive_database_names(
         repository={'path': 'repo'},
         archive='archive',
-        location={'borgmatic_source_directory': '.borgmatic'},
-        storage=flexmock(),
+        config={'borgmatic_source_directory': '.borgmatic'},
         local_borg_version=flexmock(),
+        global_arguments=flexmock(log_json=False),
         local_path=flexmock(),
         remote_path=flexmock(),
     )
@@ -115,9 +115,9 @@ def test_collect_archive_database_names_skips_bad_archive_paths():
     archive_database_names = module.collect_archive_database_names(
         repository={'path': 'repo'},
         archive='archive',
-        location={'borgmatic_source_directory': '.borgmatic'},
-        storage=flexmock(),
+        config={'borgmatic_source_directory': '.borgmatic'},
         local_borg_version=flexmock(),
+        global_arguments=flexmock(log_json=False),
         local_path=flexmock(),
         remote_path=flexmock(),
     )
@@ -148,7 +148,8 @@ def test_find_databases_to_restore_without_requested_names_finds_all_archive_dat
     archive_database_names = {'postresql_databases': ['foo', 'bar']}
 
     restore_names = module.find_databases_to_restore(
-        requested_database_names=[], archive_database_names=archive_database_names,
+        requested_database_names=[],
+        archive_database_names=archive_database_names,
     )
 
     assert restore_names == archive_database_names
@@ -158,7 +159,8 @@ def test_find_databases_to_restore_with_all_in_requested_names_finds_all_archive
     archive_database_names = {'postresql_databases': ['foo', 'bar']}
 
     restore_names = module.find_databases_to_restore(
-        requested_database_names=['all'], archive_database_names=archive_database_names,
+        requested_database_names=['all'],
+        archive_database_names=archive_database_names,
     )
 
     assert restore_names == archive_database_names
@@ -194,7 +196,9 @@ def test_ensure_databases_found_with_all_databases_found_does_not_raise():
 def test_ensure_databases_found_with_no_databases_raises():
     with pytest.raises(ValueError):
         module.ensure_databases_found(
-            restore_names={'postgresql_databases': []}, remaining_restore_names={}, found_names=[],
+            restore_names={'postgresql_databases': []},
+            remaining_restore_names={},
+            found_names=[],
         )
 
 
@@ -224,39 +228,45 @@ def test_run_restore_restores_each_database():
     ).and_return(('postgresql_databases', {'name': 'bar'}))
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'foo'},
+        database={'name': 'foo', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'bar'},
+        database={'name': 'bar', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('ensure_databases_found')
 
     module.run_restore(
         repository={'path': 'repo'},
-        location=flexmock(),
-        storage=flexmock(),
-        hooks=flexmock(),
+        config=flexmock(),
         local_borg_version=flexmock(),
-        restore_arguments=flexmock(repository='repo', archive='archive', databases=flexmock()),
+        restore_arguments=flexmock(
+            repository='repo',
+            archive='archive',
+            databases=flexmock(),
+            schemas=None,
+            hostname=None,
+            port=None,
+            username=None,
+            password=None,
+            restore_path=None,
+        ),
         global_arguments=flexmock(dry_run=False),
         local_path=flexmock(),
         remote_path=flexmock(),
@@ -274,9 +284,7 @@ def test_run_restore_bails_for_non_matching_repository():
 
     module.run_restore(
         repository={'path': 'repo'},
-        location=flexmock(),
-        storage=flexmock(),
-        hooks=flexmock(),
+        config=flexmock(),
         local_borg_version=flexmock(),
         restore_arguments=flexmock(repository='repo', archive='archive', databases=flexmock()),
         global_arguments=flexmock(dry_run=False),
@@ -298,19 +306,19 @@ def test_run_restore_restores_database_configured_with_all_name():
     flexmock(module).should_receive('collect_archive_database_names').and_return(flexmock())
     flexmock(module).should_receive('find_databases_to_restore').and_return(restore_names)
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='foo',
     ).and_return(('postgresql_databases', {'name': 'foo'}))
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='bar',
     ).and_return((None, None))
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='bar',
@@ -318,39 +326,45 @@ def test_run_restore_restores_database_configured_with_all_name():
     ).and_return(('postgresql_databases', {'name': 'bar'}))
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'foo'},
+        database={'name': 'foo', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'bar'},
+        database={'name': 'bar', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('ensure_databases_found')
 
     module.run_restore(
         repository={'path': 'repo'},
-        location=flexmock(),
-        storage=flexmock(),
-        hooks=flexmock(),
+        config=flexmock(),
         local_borg_version=flexmock(),
-        restore_arguments=flexmock(repository='repo', archive='archive', databases=flexmock()),
+        restore_arguments=flexmock(
+            repository='repo',
+            archive='archive',
+            databases=flexmock(),
+            schemas=None,
+            hostname=None,
+            port=None,
+            username=None,
+            password=None,
+            restore_path=None,
+        ),
         global_arguments=flexmock(dry_run=False),
         local_path=flexmock(),
         remote_path=flexmock(),
@@ -370,19 +384,19 @@ def test_run_restore_skips_missing_database():
     flexmock(module).should_receive('collect_archive_database_names').and_return(flexmock())
     flexmock(module).should_receive('find_databases_to_restore').and_return(restore_names)
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='foo',
     ).and_return(('postgresql_databases', {'name': 'foo'}))
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='bar',
     ).and_return((None, None))
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='bar',
@@ -390,39 +404,45 @@ def test_run_restore_skips_missing_database():
     ).and_return((None, None))
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'foo'},
+        database={'name': 'foo', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'bar'},
+        database={'name': 'bar', 'schemas': None},
+        connection_params=object,
     ).never()
     flexmock(module).should_receive('ensure_databases_found')
 
     module.run_restore(
         repository={'path': 'repo'},
-        location=flexmock(),
-        storage=flexmock(),
-        hooks=flexmock(),
+        config=flexmock(),
         local_borg_version=flexmock(),
-        restore_arguments=flexmock(repository='repo', archive='archive', databases=flexmock()),
+        restore_arguments=flexmock(
+            repository='repo',
+            archive='archive',
+            databases=flexmock(),
+            schemas=None,
+            hostname=None,
+            port=None,
+            username=None,
+            password=None,
+            restore_path=None,
+        ),
         global_arguments=flexmock(dry_run=False),
         local_path=flexmock(),
         remote_path=flexmock(),
@@ -443,52 +463,58 @@ def test_run_restore_restores_databases_from_different_hooks():
     flexmock(module).should_receive('collect_archive_database_names').and_return(flexmock())
     flexmock(module).should_receive('find_databases_to_restore').and_return(restore_names)
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='postgresql_databases',
         database_name='foo',
     ).and_return(('postgresql_databases', {'name': 'foo'}))
     flexmock(module).should_receive('get_configured_database').with_args(
-        hooks=object,
+        config=object,
         archive_database_names=object,
         hook_name='mysql_databases',
         database_name='bar',
     ).and_return(('mysql_databases', {'name': 'bar'}))
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='postgresql_databases',
-        database={'name': 'foo'},
+        database={'name': 'foo', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('restore_single_database').with_args(
         repository=object,
-        location=object,
-        storage=object,
-        hooks=object,
+        config=object,
         local_borg_version=object,
         global_arguments=object,
         local_path=object,
         remote_path=object,
         archive_name=object,
         hook_name='mysql_databases',
-        database={'name': 'bar'},
+        database={'name': 'bar', 'schemas': None},
+        connection_params=object,
     ).once()
     flexmock(module).should_receive('ensure_databases_found')
 
     module.run_restore(
         repository={'path': 'repo'},
-        location=flexmock(),
-        storage=flexmock(),
-        hooks=flexmock(),
+        config=flexmock(),
         local_borg_version=flexmock(),
-        restore_arguments=flexmock(repository='repo', archive='archive', databases=flexmock()),
+        restore_arguments=flexmock(
+            repository='repo',
+            archive='archive',
+            databases=flexmock(),
+            schemas=None,
+            hostname=None,
+            port=None,
+            username=None,
+            password=None,
+            restore_path=None,
+        ),
         global_arguments=flexmock(dry_run=False),
         local_path=flexmock(),
         remote_path=flexmock(),
