@@ -15,7 +15,7 @@ def test_run_configuration_runs_actions_for_each_repository():
     flexmock(module).should_receive('run_actions').and_return(expected_results[:1]).and_return(
         expected_results[1:]
     )
-    config = {'location': {'repositories': [{'path': 'foo'}, {'path': 'bar'}]}}
+    config = {'repositories': [{'path': 'foo'}, {'path': 'bar'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1)}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -29,7 +29,7 @@ def test_run_configuration_with_invalid_borg_version_errors():
     flexmock(module.command).should_receive('execute_hook').never()
     flexmock(module.dispatch).should_receive('call_hooks').never()
     flexmock(module).should_receive('run_actions').never()
-    config = {'location': {'repositories': ['foo']}}
+    config = {'repositories': ['foo']}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'prune': flexmock()}
 
     list(module.run_configuration('test.yaml', config, arguments))
@@ -44,7 +44,7 @@ def test_run_configuration_logs_monitor_start_error():
     expected_results = [flexmock()]
     flexmock(module).should_receive('log_error_records').and_return(expected_results)
     flexmock(module).should_receive('run_actions').never()
-    config = {'location': {'repositories': ['foo']}}
+    config = {'repositories': ['foo']}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -59,7 +59,7 @@ def test_run_configuration_bails_for_monitor_start_soft_failure():
     flexmock(module.dispatch).should_receive('call_hooks').and_raise(error)
     flexmock(module).should_receive('log_error_records').never()
     flexmock(module).should_receive('run_actions').never()
-    config = {'location': {'repositories': ['foo']}}
+    config = {'repositories': ['foo']}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -75,7 +75,7 @@ def test_run_configuration_logs_actions_error():
     expected_results = [flexmock()]
     flexmock(module).should_receive('log_error_records').and_return(expected_results)
     flexmock(module).should_receive('run_actions').and_raise(OSError)
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False)}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -91,7 +91,7 @@ def test_run_configuration_bails_for_actions_soft_failure():
     flexmock(module).should_receive('run_actions').and_raise(error)
     flexmock(module).should_receive('log_error_records').never()
     flexmock(module.command).should_receive('considered_soft_failure').and_return(True)
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -108,7 +108,7 @@ def test_run_configuration_logs_monitor_log_error():
     expected_results = [flexmock()]
     flexmock(module).should_receive('log_error_records').and_return(expected_results)
     flexmock(module).should_receive('run_actions').and_return([])
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -126,7 +126,7 @@ def test_run_configuration_bails_for_monitor_log_soft_failure():
     flexmock(module).should_receive('log_error_records').never()
     flexmock(module).should_receive('run_actions').and_return([])
     flexmock(module.command).should_receive('considered_soft_failure').and_return(True)
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -143,7 +143,7 @@ def test_run_configuration_logs_monitor_finish_error():
     expected_results = [flexmock()]
     flexmock(module).should_receive('log_error_records').and_return(expected_results)
     flexmock(module).should_receive('run_actions').and_return([])
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -161,7 +161,7 @@ def test_run_configuration_bails_for_monitor_finish_soft_failure():
     flexmock(module).should_receive('log_error_records').never()
     flexmock(module).should_receive('run_actions').and_return([])
     flexmock(module.command).should_receive('considered_soft_failure').and_return(True)
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -169,6 +169,19 @@ def test_run_configuration_bails_for_monitor_finish_soft_failure():
     assert results == []
 
 
+def test_run_configuration_does_not_call_monitoring_hooks_if_monitoring_hooks_are_disabled():
+    flexmock(module).should_receive('verbosity_to_log_level').and_return(module.DISABLED)
+    flexmock(module.borg_version).should_receive('local_borg_version').and_return(flexmock())
+
+    flexmock(module.dispatch).should_receive('call_hooks').never()
+    flexmock(module).should_receive('run_actions').and_return([])
+
+    config = {'repositories': [{'path': 'foo'}]}
+    arguments = {'global': flexmock(monitoring_verbosity=-2, dry_run=False), 'create': flexmock()}
+    results = list(module.run_configuration('test.yaml', config, arguments))
+    assert results == []
+
+
 def test_run_configuration_logs_on_error_hook_error():
     flexmock(module).should_receive('verbosity_to_log_level').and_return(logging.INFO)
     flexmock(module.borg_version).should_receive('local_borg_version').and_return(flexmock())
@@ -178,7 +191,7 @@ def test_run_configuration_logs_on_error_hook_error():
         expected_results[:1]
     ).and_return(expected_results[1:])
     flexmock(module).should_receive('run_actions').and_raise(OSError)
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -194,7 +207,7 @@ def test_run_configuration_bails_for_on_error_hook_soft_failure():
     expected_results = [flexmock()]
     flexmock(module).should_receive('log_error_records').and_return(expected_results)
     flexmock(module).should_receive('run_actions').and_raise(OSError)
-    config = {'location': {'repositories': [{'path': 'foo'}]}}
+    config = {'repositories': [{'path': 'foo'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
 
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -209,7 +222,7 @@ def test_run_configuration_retries_soft_error():
     flexmock(module.command).should_receive('execute_hook')
     flexmock(module).should_receive('run_actions').and_raise(OSError).and_return([])
     flexmock(module).should_receive('log_error_records').and_return([flexmock()]).once()
-    config = {'location': {'repositories': [{'path': 'foo'}]}, 'storage': {'retries': 1}}
+    config = {'repositories': [{'path': 'foo'}], 'retries': 1}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
     assert results == []
@@ -229,9 +242,10 @@ def test_run_configuration_retries_hard_error():
     ).and_return([flexmock()])
     error_logs = [flexmock()]
     flexmock(module).should_receive('log_error_records').with_args(
-        'foo: Error running actions for repository', OSError,
+        'foo: Error running actions for repository',
+        OSError,
     ).and_return(error_logs)
-    config = {'location': {'repositories': [{'path': 'foo'}]}, 'storage': {'retries': 1}}
+    config = {'repositories': [{'path': 'foo'}], 'retries': 1}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
     assert results == error_logs
@@ -249,7 +263,7 @@ def test_run_configuration_repos_ordered():
     flexmock(module).should_receive('log_error_records').with_args(
         'bar: Error running actions for repository', OSError
     ).and_return(expected_results[1:]).ordered()
-    config = {'location': {'repositories': [{'path': 'foo'}, {'path': 'bar'}]}}
+    config = {'repositories': [{'path': 'foo'}, {'path': 'bar'}]}
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
     assert results == expected_results
@@ -281,8 +295,8 @@ def test_run_configuration_retries_round_robin():
         'bar: Error running actions for repository', OSError
     ).and_return(bar_error_logs).ordered()
     config = {
-        'location': {'repositories': [{'path': 'foo'}, {'path': 'bar'}]},
-        'storage': {'retries': 1},
+        'repositories': [{'path': 'foo'}, {'path': 'bar'}],
+        'retries': 1,
     }
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -313,8 +327,8 @@ def test_run_configuration_retries_one_passes():
         'bar: Error running actions for repository', OSError
     ).and_return(error_logs).ordered()
     config = {
-        'location': {'repositories': [{'path': 'foo'}, {'path': 'bar'}]},
-        'storage': {'retries': 1},
+        'repositories': [{'path': 'foo'}, {'path': 'bar'}],
+        'retries': 1,
     }
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -355,8 +369,9 @@ def test_run_configuration_retry_wait():
         'foo: Error running actions for repository', OSError
     ).and_return(error_logs).ordered()
     config = {
-        'location': {'repositories': [{'path': 'foo'}]},
-        'storage': {'retries': 3, 'retry_wait': 10},
+        'repositories': [{'path': 'foo'}],
+        'retries': 3,
+        'retry_wait': 10,
     }
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -393,8 +408,9 @@ def test_run_configuration_retries_timeout_multiple_repos():
         'bar: Error running actions for repository', OSError
     ).and_return(error_logs).ordered()
     config = {
-        'location': {'repositories': [{'path': 'foo'}, {'path': 'bar'}]},
-        'storage': {'retries': 1, 'retry_wait': 10},
+        'repositories': [{'path': 'foo'}, {'path': 'bar'}],
+        'retries': 1,
+        'retry_wait': 10,
     }
     arguments = {'global': flexmock(monitoring_verbosity=1, dry_run=False), 'create': flexmock()}
     results = list(module.run_configuration('test.yaml', config, arguments))
@@ -410,11 +426,7 @@ def test_run_actions_runs_rcreate():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'rcreate': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -430,9 +442,7 @@ def test_run_actions_adds_log_file_to_hook_context():
     flexmock(borgmatic.actions.create).should_receive('run_create').with_args(
         config_filename=object,
         repository={'path': 'repo'},
-        location={'repositories': []},
-        storage=object,
-        hooks={},
+        config={'repositories': []},
         hook_context={'repository': 'repo', 'repositories': '', 'log_file': 'foo'},
         local_borg_version=object,
         create_arguments=object,
@@ -446,11 +456,7 @@ def test_run_actions_adds_log_file_to_hook_context():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'create': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -469,11 +475,7 @@ def test_run_actions_runs_transfer():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'transfer': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -492,11 +494,7 @@ def test_run_actions_runs_create():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'create': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -515,11 +513,7 @@ def test_run_actions_runs_prune():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'prune': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -537,11 +531,7 @@ def test_run_actions_runs_compact():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'compact': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -560,11 +550,7 @@ def test_run_actions_runs_check_when_repository_enabled_for_checks():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'check': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -583,11 +569,7 @@ def test_run_actions_skips_check_when_repository_not_enabled_for_checks():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'check': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -605,11 +587,7 @@ def test_run_actions_runs_extract():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'extract': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -627,11 +605,7 @@ def test_run_actions_runs_export_tar():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'export-tar': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -649,11 +623,7 @@ def test_run_actions_runs_mount():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'mount': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -671,11 +641,7 @@ def test_run_actions_runs_restore():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'restore': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -694,11 +660,7 @@ def test_run_actions_runs_rlist():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'rlist': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -718,11 +680,7 @@ def test_run_actions_runs_list():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'list': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -742,11 +700,7 @@ def test_run_actions_runs_rinfo():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'rinfo': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -766,11 +720,7 @@ def test_run_actions_runs_info():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'info': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -789,11 +739,7 @@ def test_run_actions_runs_break_lock():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'break-lock': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -811,11 +757,7 @@ def test_run_actions_runs_borg():
         module.run_actions(
             arguments={'global': flexmock(dry_run=False, log_file='foo'), 'borg': flexmock()},
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -838,11 +780,7 @@ def test_run_actions_runs_multiple_actions_in_argument_order():
                 'restore': flexmock(),
             },
             config_filename=flexmock(),
-            location={'repositories': []},
-            storage=flexmock(),
-            retention=flexmock(),
-            consistency=flexmock(),
-            hooks={},
+            config={'repositories': []},
             local_path=flexmock(),
             remote_path=flexmock(),
             local_borg_version=flexmock(),
@@ -863,7 +801,7 @@ def test_load_configurations_collects_parsed_configurations_and_logs():
     configs, logs = tuple(module.load_configurations(('test.yaml', 'other.yaml')))
 
     assert configs == {'test.yaml': configuration, 'other.yaml': other_configuration}
-    assert logs == test_expected_logs + other_expected_logs
+    assert set(logs) >= set(test_expected_logs + other_expected_logs)
 
 
 def test_load_configurations_logs_warning_for_permission_error():
@@ -872,7 +810,7 @@ def test_load_configurations_logs_warning_for_permission_error():
     configs, logs = tuple(module.load_configurations(('test.yaml',)))
 
     assert configs == {}
-    assert {log.levelno for log in logs} == {logging.WARNING}
+    assert max(log.levelno for log in logs) == logging.WARNING
 
 
 def test_load_configurations_logs_critical_for_parse_error():
@@ -881,7 +819,7 @@ def test_load_configurations_logs_critical_for_parse_error():
     configs, logs = tuple(module.load_configurations(('test.yaml',)))
 
     assert configs == {}
-    assert {log.levelno for log in logs} == {logging.CRITICAL}
+    assert max(log.levelno for log in logs) == logging.CRITICAL
 
 
 def test_log_record_does_not_raise():
@@ -900,7 +838,19 @@ def test_log_error_records_generates_output_logs_for_message_only():
     assert {log['levelno'] for log in logs} == {logging.CRITICAL}
 
 
-def test_log_error_records_generates_output_logs_for_called_process_error():
+def test_log_error_records_generates_output_logs_for_called_process_error_with_bytes_ouput():
+    flexmock(module).should_receive('log_record').replace_with(dict)
+    flexmock(module.logger).should_receive('getEffectiveLevel').and_return(logging.WARNING)
+
+    logs = tuple(
+        module.log_error_records('Error', subprocess.CalledProcessError(1, 'ls', b'error output'))
+    )
+
+    assert {log['levelno'] for log in logs} == {logging.CRITICAL}
+    assert any(log for log in logs if 'error output' in str(log))
+
+
+def test_log_error_records_generates_output_logs_for_called_process_error_with_string_ouput():
     flexmock(module).should_receive('log_record').replace_with(dict)
     flexmock(module.logger).should_receive('getEffectiveLevel').and_return(logging.WARNING)
 
@@ -937,15 +887,145 @@ def test_log_error_records_generates_nothing_for_other_error():
 
 
 def test_get_local_path_uses_configuration_value():
-    assert module.get_local_path({'test.yaml': {'location': {'local_path': 'borg1'}}}) == 'borg1'
-
-
-def test_get_local_path_without_location_defaults_to_borg():
-    assert module.get_local_path({'test.yaml': {}}) == 'borg'
+    assert module.get_local_path({'test.yaml': {'local_path': 'borg1'}}) == 'borg1'
 
 
 def test_get_local_path_without_local_path_defaults_to_borg():
-    assert module.get_local_path({'test.yaml': {'location': {}}}) == 'borg'
+    assert module.get_local_path({'test.yaml': {}}) == 'borg'
+
+
+def test_collect_highlander_action_summary_logs_info_for_success_with_bootstrap():
+    flexmock(module.borg_version).should_receive('local_borg_version').and_return(flexmock())
+    flexmock(module.borgmatic.actions.config.bootstrap).should_receive('run_bootstrap')
+    arguments = {
+        'bootstrap': flexmock(repository='repo'),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+    assert {log.levelno for log in logs} == {logging.ANSWER}
+
+
+def test_collect_highlander_action_summary_logs_error_on_bootstrap_failure():
+    flexmock(module.borg_version).should_receive('local_borg_version').and_return(flexmock())
+    flexmock(module.borgmatic.actions.config.bootstrap).should_receive('run_bootstrap').and_raise(
+        ValueError
+    )
+    arguments = {
+        'bootstrap': flexmock(repository='repo'),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+
+    assert {log.levelno for log in logs} == {logging.CRITICAL}
+
+
+def test_collect_highlander_action_summary_logs_error_on_bootstrap_local_borg_version_failure():
+    flexmock(module.borg_version).should_receive('local_borg_version').and_raise(ValueError)
+    flexmock(module.borgmatic.actions.config.bootstrap).should_receive('run_bootstrap').never()
+    arguments = {
+        'bootstrap': flexmock(repository='repo'),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+
+    assert {log.levelno for log in logs} == {logging.CRITICAL}
+
+
+def test_collect_highlander_action_summary_logs_info_for_success_with_generate():
+    flexmock(module.borgmatic.actions.config.generate).should_receive('run_generate')
+    arguments = {
+        'generate': flexmock(destination='test.yaml'),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+    assert {log.levelno for log in logs} == {logging.ANSWER}
+
+
+def test_collect_highlander_action_summary_logs_error_on_generate_failure():
+    flexmock(module.borgmatic.actions.config.generate).should_receive('run_generate').and_raise(
+        ValueError
+    )
+    arguments = {
+        'generate': flexmock(destination='test.yaml'),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+
+    assert {log.levelno for log in logs} == {logging.CRITICAL}
+
+
+def test_collect_highlander_action_summary_logs_info_for_success_with_validate():
+    flexmock(module.borgmatic.actions.config.validate).should_receive('run_validate')
+    arguments = {
+        'validate': flexmock(),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+    assert {log.levelno for log in logs} == {logging.ANSWER}
+
+
+def test_collect_highlander_action_summary_logs_error_on_validate_parse_failure():
+    flexmock(module.borgmatic.actions.config.validate).should_receive('run_validate')
+    arguments = {
+        'validate': flexmock(),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=True
+        )
+    )
+
+    assert {log.levelno for log in logs} == {logging.CRITICAL}
+
+
+def test_collect_highlander_action_summary_logs_error_on_run_validate_failure():
+    flexmock(module.borgmatic.actions.config.validate).should_receive('run_validate').and_raise(
+        ValueError
+    )
+    arguments = {
+        'validate': flexmock(),
+        'global': flexmock(dry_run=False),
+    }
+
+    logs = tuple(
+        module.collect_highlander_action_summary_logs(
+            {'test.yaml': {}}, arguments=arguments, configuration_parse_errors=False
+        )
+    )
+
+    assert {log.levelno for log in logs} == {logging.CRITICAL}
 
 
 def test_collect_configuration_run_summary_logs_info_for_success():
